@@ -7,8 +7,10 @@ import enum
 import ipaddress
 import math
 import numbers
+import sys
 from typing import (
     Any,
+    Dict,
     List,
     Literal,
     Mapping,
@@ -21,9 +23,12 @@ from typing import (
     TypedDict,
     TYPE_CHECKING,
     Union,
+    cast,
 )
 import uuid
 
+import typeguard
+
 __all__ = (
     'aobject',
     'DeviceId',
@@ -42,6 +47,7 @@ __all__ = (
     'SlotName',
     'IntrinsicSlotNames',
     'ResourceSlot',
+    'HardwareMetadata',
     'MountPermission',
     'MountPermissionLiteral',
     'MountTuple5',
@@ -51,6 +57,10 @@ __all__ = (
     'KernelCreationConfig',
     'KernelCreationResult',
     'ServicePortProtocols',
+    'ClusterInfo',
+    'ClusterMode',
+    'ClusterSSHKeyPair',
+    'check_typed_dict',
 )
 
 if TYPE_CHECKING:
@@ -99,6 +109,26 @@ class aobject(object):
         pass
 
 
+TD = TypeVar('TD')
+
+
+def check_typed_dict(value: Mapping[Any, Any], expected_type: Type[TD]) -> TD:
+    """
+    Validates the given dict against the given TypedDict class, and wraps the value as the given TypedDict type.
+
+    This is a shortcut to :func:`typeguard.check_typed_dict()` function to fill extra information
+    """
+    assert issubclass(expected_type, dict) and hasattr(expected_type, '__annotations__'), \
+           f"expected_type ({type(expected_type)}) must be a TypedDict class"
+    frame = sys._getframe(1)
+    _globals = frame.f_globals
+    _locals = frame.f_locals
+    memo = typeguard._TypeCheckMemo(_globals, _locals)
+    typeguard.check_typed_dict('value', value, expected_type, memo)
+    # Here we passed the check, so return it after casting.
+    return cast(TD, value)
+
+
 PID = NewType('PID', int)
 HostPID = NewType('HostPID', PID)
 ContainerPID = NewType('ContainerPID', PID)
@@ -123,6 +153,12 @@ class SlotTypes(str, enum.Enum):
     UNIQUE = 'unique'
 
 
+class HardwareMetadata(TypedDict):
+    status: Literal["healthy", "degraded", "offline", "unavailable"]
+    status_info: Optional[str]
+    metadata: Dict[str, str]
+
+
 class AutoPullBehavior(str, enum.Enum):
     DIGEST = 'digest'
     TAG = 'tag'
@@ -146,6 +182,11 @@ class SessionResult(str, enum.Enum):
     FAILURE = 'failure'
 
 
+class ClusterMode(str, enum.Enum):
+    SINGLE_NODE = 'single-node'
+    MULTI_NODE = 'multi-node'
+
+
 class MovingStatValue(TypedDict):
     min: str
     max: str
@@ -587,6 +628,19 @@ class ServicePort(TypedDict):
     host_ports: Sequence[Optional[int]]
 
 
+class ClusterInfo(TypedDict):
+    mode: ClusterMode
+    size: int
+    replicas: Mapping[str, int]  # per-role kernel counts
+    network_name: Optional[str]
+    ssh_keypair: Optional[ClusterSSHKeyPair]
+
+
+class ClusterSSHKeyPair(TypedDict):
+    public_key: str   # OpenSSH authorized-keys compatible format
+    private_key: str  # PEM-encoded string
+
+
 class DeviceModelInfo(TypedDict):
     device_id: DeviceId
     model_name: str
@@ -610,7 +664,10 @@ class KernelCreationConfig(TypedDict):
     image: ImageConfig
     auto_pull: AutoPullBehavior
     session_type: SessionTypes
-    cluster: dict
+    cluster_mode: ClusterMode
+    cluster_role: str       # the kernel's role in the cluster
+    cluster_idx: int        # the kernel's index in the cluster
+    cluster_hostname: str   # the kernel's hostname in the cluster
     resource_slots: Mapping[str, str]  # json form of ResourceSlot
     resource_opts: Mapping[str, str]   # json form of resource options
     environ: Mapping[str, str]
@@ -627,7 +684,8 @@ class KernelCreationConfig(TypedDict):
 class KernelEnqueueingConfig(TypedDict):
     image_ref: ImageRef
     cluster_role: str
-    idx: int
+    cluster_idx: int
+    cluster_hostname: str
     creation_config: dict
     bootstrap_script: str
     startup_command: str
